﻿Imports Data.Tsql

<Serializable()> _
Public Class OrderByCollection
    Inherits System.Collections.CollectionBase

#Region "Constructors"
    ''' <summary>
    ''' Creates a new <see cref="OrderByCollection"/> object.
    ''' </summary>
    Public Sub New()
        MyBase.New()
    End Sub

    ''' <summary>
    ''' Creates a new <see cref="OrderByCollection"/> object from a sort criteria 
    ''' string.
    ''' </summary>
    Public Sub New(ByVal strSortCriteria As String)
        MyBase.New()
        Me.Parse(strSortCriteria)
    End Sub

    ''' <summary>
    ''' Creates a new <see cref="OrderByCollection"/> object from a 
    ''' <c>SortedColsCollection</c> object.
    ''' </summary>
    Public Sub New(ByVal sccSortedColsCollection As Infragistics.Web.UI.GridControls.SortedColumnInfoCollection)
        MyBase.New()
        Me.Parse(sccSortedColsCollection)
    End Sub
#End Region

#Region "Properties"
    ''' <summary>
    ''' Returns a specific member of a <see cref="OrderByCollection"/> object.
    ''' </summary>
    Public ReadOnly Property Item(ByVal nIndex As Integer) As OrderBy
        Get
            Return CType(Me.List.Item(nIndex), OrderBy)
        End Get
    End Property
#End Region

#Region "Methods"
    ''' <summary>
    ''' Adds a <see cref="OrderBy"/> object to a <see cref="OrderByCollection"/>.
    ''' </summary>
    Public Sub Add(ByVal orderBy As OrderBy)
        Me.EnsureUniqueColumnName(orderBy.ColumnName)
        Me.List.Add(orderBy)
    End Sub

    ''' <summary>
    ''' Determines whether the <see cref="OrderByCollection"/> contains a specific 
    ''' <see cref="OrderBy"/> object.
    ''' </summary>
    Public Function Contains(ByVal orderBy As OrderBy) As Boolean
        Return Me.List.Contains(orderBy)
    End Function

    ''' <summary>
    ''' Copies the elements of the <see cref="OrderByCollection"/> to an <c>Array</c>, 
    ''' starting at a particular <c>Array</c> index.
    ''' </summary>
    Public Sub CopyTo(ByVal arrSortCriteria() As OrderBy, ByVal nIndex As Integer)
        Me.CopyTo(arrSortCriteria, nIndex)
    End Sub

    Public Sub EnsureUniqueColumnName(ByVal columnName As String)
        If (Me.IndexOf(columnName) > -1) Then
            Throw New DuplicateNameException()
        End If
    End Sub

    ''' <summary>
    ''' Inserts a <see cref="OrderBy"/> object into a <see cref="OrderByCollection"/>.
    ''' </summary>
    Public Sub Insert(ByVal index As Integer, ByVal orderBy As OrderBy)
        Me.EnsureUniqueColumnName(orderBy.ColumnName)
        Me.List.Insert(index, orderBy)
    End Sub

    ''' <summary>
    ''' Initialises a <see cref="OrderByCollection"/> object with an SQL sort criteria 
    ''' string.
    ''' </summary>
    Public Sub FromSql(ByVal strSortCriteria As String)
        Me.Clear()
        If (strSortCriteria Is Nothing) Then
            Exit Sub
        End If

        If (strSortCriteria.Length = 0) Then
            Exit Sub
        End If

        Dim strParts() As String = strSortCriteria.Split(Keyword.FieldDelimiter)
        Dim nPart As Integer
        For nPart = 0 To strParts.Length - 1
            Dim scSortCriteria As New OrderBy
            scSortCriteria.FromSql(strParts(nPart))
            Add(scSortCriteria)
        Next
    End Sub

    ''' <summary>
    ''' Determines the index of a specific item in the <see cref="OrderByCollection"/>.
    ''' </summary>
    Public Function IndexOf(ByVal orderBy As OrderBy) As Integer
        Return Me.List.IndexOf(orderBy)
    End Function

    ''' <summary>
    ''' Determines the index of a specific item in the <see cref="OrderByCollection"/>.
    ''' </summary>
    ''' <param name="columnName">Name of the column.</param>
    ''' <returns>The index of <paramref name="columnName" /> if found in the 
    ''' <see cref="OrderByCollection"/>; otherwise, -1.</returns>
    Public Function IndexOf(ByVal columnName As String) As Integer

        For n As Integer = 0 To Me.List.Count - 1
            Dim orderBy As OrderBy = Me.Item(n)
            If (orderBy.ColumnName.Equals(columnName, StringComparison.OrdinalIgnoreCase)) Then
                Return n
            End If
        Next

        Return -1

    End Function

    Public Sub Parse(ByVal strSortCriteria As String)
        If (strSortCriteria Is Nothing) Then
            Exit Sub
        End If

        Me.Clear()

        If (strSortCriteria.Length = 0) Then
            Exit Sub
        End If

        Dim strParts() As String = strSortCriteria.Split(Keyword.FieldDelimiter)
        Dim nPart As Integer
        For nPart = 0 To strParts.Length - 1
            Dim scSortCriteria As New OrderBy(strParts(nPart).Trim())
            Me.Add(scSortCriteria)
        Next
    End Sub

    Public Sub Parse(ByVal sccSortedColsCollection As Infragistics.Web.UI.GridControls.SortedColumnInfoCollection)

        If IsNothing(sccSortedColsCollection) Then
            Throw New ArgumentNullException("sccSortedColsCollection")
        End If

        Me.Clear()

        Dim ugcColumn As Infragistics.Web.UI.GridControls.SortedColumnInfo

        For Each ugcColumn In sccSortedColsCollection
            Dim scSortCriteria As OrderBy = New OrderBy(ugcColumn)
            Me.Add(scSortCriteria)
        Next

    End Sub

    ''' <summary>
    ''' Removes a <see cref="OrderBy"/> object from a <see cref="OrderByCollection"/>.
    ''' </summary>
    Public Sub Remove(ByVal nIndex As Integer)
        Me.List.RemoveAt(nIndex)
    End Sub

    ''' <summary>
    ''' Gets a human readable sort criteria string from a 
    ''' <see cref="OrderByCollection"/>.
    ''' </summary>
    Public Overrides Function ToString() As String

        Dim strCriteria As String = ""
        Dim strSortCriteria As String = ""
        Dim scSortCriteria As OrderBy

        For Each scSortCriteria In Me.List
            strCriteria = scSortCriteria.ToString
            If Not (strCriteria.Length = 0) Then
                If (strSortCriteria <> "") Then
                    strSortCriteria = String.Concat(strSortCriteria, Keyword.FieldDelimiter)
                End If
                strSortCriteria = String.Concat(strSortCriteria, strCriteria)
            End If
        Next
        Return strSortCriteria

    End Function

    ''' <summary>
    ''' Gets an SQL sort criteria string from a <see cref="OrderByCollection"/>.
    ''' </summary>
    Public Function ToSql() As String

        Dim strCriteria As String
        Dim strSql As String = Nothing
        Dim scSortCriteria As OrderBy

        If (Me.List.Count = 0) Then
            Return String.Empty
        End If

        For Each scSortCriteria In Me.List
            strCriteria = scSortCriteria.ToSql
            If Not (strCriteria.Length = 0) Then
                If (strSql <> "") Then
                    strSql = String.Concat(strSql, Keyword.FieldDelimiter)
                End If
                strSql = String.Concat(strSql, strCriteria)
            End If
        Next
        Return strSql

    End Function

    ''' <summary>
    ''' Tries to add an <see cref="OrderBy"/> to the <see cref="OrderByCollection"/>.
    ''' </summary>
    Public Function TryAdd(ByVal orderBy As OrderBy) As Boolean

        Try
            Me.Add(orderBy)
        Catch ex As Exception
            Return False
        End Try

        Return True

    End Function

    ''' <summary>
    ''' Tries to insert an <see cref="OrderBy"/> into the <see cref="OrderByCollection"/>.
    ''' </summary>
    Public Function TryInsert(ByVal index As Integer, ByVal orderBy As OrderBy) As Boolean

        Try
            Me.Insert(index, orderBy)
        Catch ex As Exception
            Return False
        End Try

        Return True

    End Function
#End Region
End Class
